#include "stdafx.h"

#include <stdio.h>
#include <string.h>

#include "OWLIFTViewDlg.h"

COWLIFTViewDlg dlg;


COWLIFTViewDlg::COWLIFTViewDlg() :
	m_owDev(NULL),
	m_dcCreated(FALSE),
	m_decodeBuf(NULL),
	m_tempTab(NULL),
	m_closing(FALSE)
{
	InitializeCriticalSection(&m_lock);
}


COWLIFTViewDlg::~COWLIFTViewDlg()
{
	Release();
}


void COWLIFTViewDlg::ShowError(OwStatus_t st)
{
	WCHAR msg[256];
	size_t sz;

	mbstowcs_s(&sz, msg, OwLib_GetErrorMessage(st), ARRAYSIZE(msg));

	MessageBox(m_hWnd, msg, L"Error", MB_ICONERROR | MB_OK);
}


BOOL COWLIFTViewDlg::OnInitDialog()
{
	GetDlgItem(IDC_START).EnableWindow(TRUE);
	GetDlgItem(IDC_STOP).EnableWindow(FALSE);

	return TRUE;
}


void COWLIFTViewDlg::Release()
{
	if (m_owDev) {
		OwLib_CaptureStop(m_owDev);
		OwLib_Release(m_owDev);
		m_owDev = NULL;
	}

	::DeleteDC(m_dc);

	if (m_hbmp) {
		::DeleteObject(m_hbmp);
		m_hbmp = NULL;
	}
}


void COWLIFTViewDlg::CaptureProcStatic(OwDev dev, BYTE *buf, UINT32 bufLen, void *userData)
{
	((COWLIFTViewDlg *)userData)->CaptureProc(buf, bufLen);
}


void COWLIFTViewDlg::SetLabelTempK(int id, UINT16 temp)
{
	WCHAR str[32];

	swprintf_s(str, L"%.2lfK", (double)temp / 100);
	GetDlgItem(id).SetWindowText(str);
}


void COWLIFTViewDlg::SetLabelUINT32(int id, UINT32 v)
{
	WCHAR str[32];

	swprintf_s(str, L"%d", v);
	GetDlgItem(id).SetWindowText(str);
}


void COWLIFTViewDlg::CaptureProc(BYTE *buf, UINT32 bufLen)
{
	HDC dlgDC = ::GetDC(m_hWnd);

	if (!m_dcCreated) {

		m_dc = ::CreateCompatibleDC(dlgDC);

		BITMAPINFOHEADER bih;

		ZeroMemory(&bih, sizeof(bih));
		bih.biSize = sizeof(bih);
		bih.biWidth = m_fx;
		bih.biHeight = m_fy;
		bih.biPlanes = 1;
		bih.biBitCount = 24;
		bih.biCompression = BI_RGB;
		m_hbmp = ::CreateDIBSection(m_dc, (BITMAPINFO *)&bih,
				DIB_PAL_COLORS, (void **)&m_decodeBuf, NULL, 0);
		::SelectObject(m_dc, m_hbmp);
		::SetStretchBltMode(dlgDC, HALFTONE);
		::GetClientRect(m_hWnd, &m_prevRect);

		LOGFONT lf;

		ZeroMemory(&lf, sizeof(lf));
		lf.lfHeight = 20;
		lf.lfWeight = lf.lfHeight * 20;
		wcscpy_s(lf.lfFaceName, L"Arial");
		m_hfont = ::CreateFontIndirect(&lf);

		m_dcCreated = TRUE;
	}

	const int padx = 8, pady = 8;

	if (OwLib_Decode(m_owDev, buf, m_decodeBuf, m_fx * m_fy * 3,
		0, OWDECODETYPE_AGC_BLUE_ORANGE_24) != OWST_OK) {
		return;
	}

	OwLib_GetTempTable(m_owDev, m_tempTab, m_fx * m_fy * sizeof(WORD));
	OwLib_FinishDecode(m_owDev);

	// draw marker
	int tx = m_fx / 2;
	int ty = m_fy / 2;

	HPEN hpen = (HPEN)::GetStockObject(WHITE_PEN);
	HGDIOBJ oldPen = ::SelectObject(m_dc, hpen);
	::Rectangle(m_dc, tx - 2, ty, tx + 3, ty + 1);
	::Rectangle(m_dc, tx, ty - 2, tx + 1, ty + 3);
	::SelectObject(m_dc, oldPen);

	// draw image
	::StretchBlt(dlgDC,
		m_prevRect.left + padx, m_prevRect.top + pady,
		m_fx * m_mag, m_fy * m_mag,
		m_dc, 0, m_fy - 1, m_fx, -m_fy, SRCCOPY);

	// marker's temperature
	double temp = (double)m_tempTab[ty * m_fx + tx] / OwLib_GetLastTempResolution(m_owDev) - 273.15;
	WCHAR temps[32];

	swprintf_s(temps, L"%.2lfC", temp);

	HGDIOBJ oldFont = ::SelectObject(dlgDC, m_hfont);
	int oldBkMode = ::SetBkMode(dlgDC, TRANSPARENT);
	::SetTextColor(dlgDC, RGB(255, 255, 255));
	::TextOut(dlgDC, (tx + 1) * m_mag + padx, (ty + 1) * m_mag + pady,
		temps, wcsnlen_s(temps, ARRAYSIZE(temps)));
	::SetBkMode(dlgDC, oldBkMode);
	::SelectObject(dlgDC, oldFont);

	// telemetry data
	LepTelemetryDataLineA *telem = OwLib_GetTelemetryData(m_owDev, buf);

	UINT16 fpaTemp = VOSPI16_TO_UINT16(telem->fpaTemp_Kx100);
	UINT16 housingTemp = VOSPI16_TO_UINT16(telem->housingTemp_Kx100);
	UINT32 frameCounter = VOSPI32_TO_UINT32(telem->frameCounter);

	EnterCriticalSection(&m_lock);

	if (!m_closing) {
		SetLabelUINT32(IDC_FRAME_COUNTER, frameCounter);
		SetLabelTempK(IDC_FPA_TEMP, fpaTemp);
		SetLabelTempK(IDC_HOUSING_TEMP, housingTemp);
	}

	LeaveCriticalSection(&m_lock);
}


void COWLIFTViewDlg::OnBnClickedStart()
{
	OwDev *owDevs;
	int numDevs;
	OwHostErrorCode_t he;
	OwStatus_t st;

	if ((st = OwLib_GetDevices(&owDevs, &numDevs, &he)) != OWST_OK) {
		GetDlgItem(IDC_START).EnableWindow(FALSE);
		GetDlgItem(IDC_FFC_MODE).EnableWindow(FALSE);
		GetDlgItem(IDC_RUN_FFC).EnableWindow(FALSE);
		ShowError(st);
		return;
	}

	m_owDev = owDevs[0];
	OwLib_ReleaseArray(owDevs);

	m_fx = OwLib_GetFrameWidth(m_owDev);
	m_fy = OwLib_GetFrameHeight(m_owDev);
	m_tempTab = new WORD[m_fx * m_fy];
	m_mag = m_fx > 80 ? 2 : 4;

	if ((st = OwLib_CaptureSetup(m_owDev, OWFRAMERATE_DEFAULT, CaptureProcStatic, this))
		!= OWST_OK) {
		ShowError(st);
		return;
	}

	if ((st = OwLib_CaptureStart(m_owDev)) != OWST_OK) {
		ShowError(st);
		return;
	}

	GetDlgItem(IDC_START).EnableWindow(FALSE);
	GetDlgItem(IDC_STOP).EnableWindow(TRUE);
}


void COWLIFTViewDlg::OnBnClickedStop()
{
	OwLib_CaptureStop(m_owDev);

	GetDlgItem(IDC_START).EnableWindow(TRUE);
	GetDlgItem(IDC_STOP).EnableWindow(FALSE);
}


void COWLIFTViewDlg::OnBnClickedFfcMode()
{
	OwStatus_t st;
	UINT16 data[16];
	char str[256];
	int i;

	if (m_owDev == NULL)
		return;

	if ((st = OwLib_CommandGet(m_owDev, 0x0200, 0x3C, data, ARRAYSIZE(data)))
		!= OWST_OK) {
		ShowError(st);
	}

	str[0] = 0;

	for (i = 0; i < ARRAYSIZE(data); ++i) {
		char tmp[32];

		sprintf_s(tmp, "%04X ", data[i]);
		strcat_s(str, tmp);

		if (i % 4 == 3)
			strcat_s(str, "\n");
	}

	MessageBoxA(m_hWnd, str, "FFC Mode", MB_OK);
}


void COWLIFTViewDlg::OnBnClickedRunFfc()
{
	OwStatus_t st;

	if (m_owDev == NULL)
		return;

	if ((st = OwLib_CommandRun(m_owDev, 0x0200, 0x40)) != OWST_OK) {
		ShowError(st);
	}
}


void COWLIFTViewDlg::OnBnClickedExit()
{
	::DestroyWindow(m_hWnd);
}


void COWLIFTViewDlg::OnDestroy()
{
	EnterCriticalSection(&m_lock);
	m_closing = TRUE;
	LeaveCriticalSection(&m_lock);
	Release();
}


static INT_PTR CALLBACK DialogProcStatic(HWND hWnd, UINT uMsg,
	WPARAM wParam, LPARAM lParam)
{
	return (INT_PTR)dlg.DialogProc(hWnd, uMsg, wParam, lParam);
}


LRESULT COWLIFTViewDlg::DialogProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		OnInitDialog();
		break;

	case WM_CLOSE:
		::DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		::PostQuitMessage(0);
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_EXIT:
			OnBnClickedExit();
			break;

		case IDC_START:
			OnBnClickedStart();
			break;

		case IDC_STOP:
			OnBnClickedStop();
			break;

		case IDC_FFC_MODE:
			OnBnClickedFfcMode();
			break;

		case IDC_RUN_FFC:
			OnBnClickedRunFfc();
			break;

		default:
			return FALSE;
		}
		break;

	default:
		return FALSE;
	}

	return TRUE;
}


CSimpleWnd COWLIFTViewDlg::GetDlgItem(int id)
{
	return CSimpleWnd(::GetDlgItem(m_hWnd, id));
}


void COWLIFTViewDlg::ShowWindow()
{
	MSG msg;
	int ret;

	::ShowWindow(m_hWnd, SW_NORMAL);

	while ((ret = ::GetMessage(&msg, NULL, 0, 0)) != 0 && ret != -1) {
		if (!::IsDialogMessage(m_hWnd, &msg)) {
			::TranslateMessage(&msg);
			::DispatchMessage(&msg);
		}
	}
}


BOOL COWLIFTViewDlg::Create(HINSTANCE hInst)
{
	m_hInst = hInst;

	if ((m_hWnd = ::CreateDialogParam(hInst,
		MAKEINTRESOURCE(IDD_OWLIFTVIEW_DIALOG),
		0, DialogProcStatic, 0)) == NULL) {

		return FALSE;
	}

	return TRUE;
}


////////////////////////////
// CSimpleWnd

void CSimpleWnd::EnableWindow(BOOL b)
{
	::EnableWindow(m_hWnd, b);
}


void CSimpleWnd::SetWindowText(LPCWSTR str)
{
	::SetWindowText(m_hWnd, str);
}


////////////////////////////
// WinMain

int APIENTRY WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpszCmdLine, int nCmdShow)
{
	if (!dlg.Create(hInst)) {
		::MessageBox(NULL, L"FaLled to create dialog", L"OWLIFTView",
			MB_APPLMODAL | MB_ICONSTOP);
		return FALSE;
	}

	dlg.ShowWindow();

	return 0;
}
